@@ -16,6 +16,7 @@
 
 """Policy Engine For Glance"""
 
+from collections import abc
 import copy
 
 from oslo_config import cfg
@@ -25,33 +26,26 @@ from oslo_policy import policy
 from glance.common import exception
 import glance.domain.proxy
 from glance.i18n import _
+from glance import policies
 
 
 LOG = logging.getLogger(__name__)
 CONF = cfg.CONF
-
-DEFAULT_RULES = policy.Rules.from_dict({
-    'context_is_admin': 'role:admin',
-    'default': '@',
-    'manage_image_cache': 'role:admin',
-})
+_ENFORCER = None
 
 
 class Enforcer(policy.Enforcer):
     """Responsible for loading and enforcing rules"""
 
     def __init__(self):
-        if CONF.find_file(CONF.oslo_policy.policy_file):
-            kwargs = dict(rules=None, use_conf=True)
-        else:
-            kwargs = dict(rules=DEFAULT_RULES, use_conf=False)
-        super(Enforcer, self).__init__(CONF, overwrite=False, **kwargs)
+        super(Enforcer, self).__init__(CONF, use_conf=True, overwrite=False)
+        self.register_defaults(policies.list_rules())
 
     def add_rules(self, rules):
         """Add new rules to the Rules object"""
         self.set_rules(rules, overwrite=False, use_conf=self.use_conf)
 
-    def enforce(self, context, action, target):
+    def enforce(self, context, action, target, registered=True):
         """Verifies that the action is valid on the target in this context.
 
            :param context: Glance request context
@@ -60,13 +54,15 @@ class Enforcer(policy.Enforcer):
            :raises: `glance.common.exception.Forbidden`
            :returns: A non-False value if access is allowed.
         """
+        if registered and action not in self.registered_rules:
+            raise policy.PolicyNotRegistered(action)
         return super(Enforcer, self).enforce(action, target,
                                              context.to_policy_values(),
                                              do_raise=True,
                                              exc=exception.Forbidden,
                                              action=action)
 
-    def check(self, context, action, target):
+    def check(self, context, action, target, registered=True):
         """Verifies that the action is valid on the target in this context.
 
            :param context: Glance request context
@@ -74,6 +70,8 @@ class Enforcer(policy.Enforcer):
            :param target: Dictionary representing the object of the action.
            :returns: A non-False value if access is allowed.
         """
+        if registered and action not in self.registered_rules:
+            raise policy.PolicyNotRegistered(action)
         return super(Enforcer, self).enforce(action,
                                              target,
                                              context.to_policy_values())
@@ -88,6 +86,14 @@ class Enforcer(policy.Enforcer):
         return self.check(context, 'context_is_admin', context.to_dict())
 
 
+def get_enforcer():
+    CONF([], project='glance')
+    global _ENFORCER
+    if _ENFORCER is None:
+        _ENFORCER = Enforcer()
+    return _ENFORCER
+
+
 class ImageRepoProxy(glance.domain.proxy.Repo):
 
     def __init__(self, image_repo, context, policy):
@@ -106,7 +112,8 @@ class ImageRepoProxy(glance.domain.proxy.Repo):
             self.policy.enforce(self.context, 'get_image', {})
             raise
         else:
-            self.policy.enforce(self.context, 'get_image', ImageTarget(image))
+            self.policy.enforce(self.context, 'get_image',
+                                dict(ImageTarget(image)))
         return image
 
     def list(self, *args, **kwargs):
@@ -114,14 +121,21 @@ class ImageRepoProxy(glance.domain.proxy.Repo):
         return super(ImageRepoProxy, self).list(*args, **kwargs)
 
     def save(self, image, from_state=None):
-        self.policy.enforce(self.context, 'modify_image', image.target)
+        self.policy.enforce(self.context, 'modify_image', dict(image.target))
         return super(ImageRepoProxy, self).save(image, from_state=from_state)
 
     def add(self, image):
-        self.policy.enforce(self.context, 'add_image', image.target)
+        self.policy.enforce(self.context, 'add_image', dict(image.target))
         return super(ImageRepoProxy, self).add(image)
 
 
+def _enforce_image_visibility(policy, context, visibility, target):
+    if visibility == 'public':
+        policy.enforce(context, 'publicize_image', target)
+    elif visibility == 'community':
+        policy.enforce(context, 'communitize_image', target)
+
+
 class ImageProxy(glance.domain.proxy.Image):
 
     def __init__(self, image, context, policy):
@@ -137,8 +151,8 @@ class ImageProxy(glance.domain.proxy.Image):
 
     @visibility.setter
     def visibility(self, value):
-        if value == 'public':
-            self.policy.enforce(self.context, 'publicize_image', self.target)
+        _enforce_image_visibility(self.policy, self.context, value,
+                                  self.target)
         self.image.visibility = value
 
     @property
@@ -159,7 +173,7 @@ class ImageProxy(glance.domain.proxy.Image):
         self.image.locations = new_locations
 
     def delete(self):
-        self.policy.enforce(self.context, 'delete_image', self.target)
+        self.policy.enforce(self.context, 'delete_image', dict(self.target))
         return self.image.delete()
 
     def deactivate(self):
@@ -181,7 +195,6 @@ class ImageProxy(glance.domain.proxy.Image):
         return self.image.get_data(*args, **kwargs)
 
     def set_data(self, *args, **kwargs):
-        self.policy.enforce(self.context, 'upload_image', self.target)
         return self.image.set_data(*args, **kwargs)
 
 
@@ -206,8 +219,8 @@ class ImageFactoryProxy(glance.domain.proxy.ImageFactory):
                                                 proxy_kwargs=proxy_kwargs)
 
     def new_image(self, **kwargs):
-        if kwargs.get('visibility') == 'public':
-            self.policy.enforce(self.context, 'publicize_image', {})
+        _enforce_image_visibility(self.policy, self.context,
+                                  kwargs.get('visibility'), {})
         return super(ImageFactoryProxy, self).new_image(**kwargs)
 
 
@@ -371,7 +384,7 @@ class TaskFactoryProxy(glance.domain.proxy.TaskFactory):
             task_proxy_kwargs=proxy_kwargs)
 
 
-class ImageTarget(object):
+class ImageTarget(abc.Mapping):
     SENTINEL = object()
 
     def __init__(self, target):
@@ -380,6 +393,9 @@ class ImageTarget(object):
         :param target: Object being targeted
         """
         self.target = target
+        self._target_keys = [k for k in dir(ImageProxy)
+                             if not k.startswith('__')
+                             if not callable(getattr(ImageProxy, k))]
 
     def __getitem__(self, key):
         """Return the value of 'key' from the target.
@@ -399,6 +415,23 @@ class ImageTarget(object):
                 value = None
         return value
 
+    def get(self, key, default=None):
+        try:
+            return self.__getitem__(key)
+        except KeyError:
+            return default
+
+    def __len__(self):
+        length = len(self._target_keys)
+        length += len(getattr(self.target, 'extra_properties', {}))
+        return length
+
+    def __iter__(self):
+        for key in self._target_keys:
+            yield key
+        for key in getattr(self.target, 'extra_properties', {}).keys():
+            yield key
+
     def key_transforms(self, key):
         if key == 'id':
             key = 'image_id'
@@ -415,6 +448,10 @@ class MetadefNamespaceProxy(glance.domain.proxy.MetadefNamespace):
         self.policy = policy
         super(MetadefNamespaceProxy, self).__init__(namespace)
 
+    def delete(self):
+        self.policy.enforce(self.context, 'delete_metadef_namespace', {})
+        return super(MetadefNamespaceProxy, self).delete()
+
 
 class MetadefNamespaceRepoProxy(glance.domain.proxy.MetadefNamespaceRepo):
 
@@ -444,6 +481,14 @@ class MetadefNamespaceRepoProxy(glance.domain.proxy.MetadefNamespaceRepo):
         self.policy.enforce(self.context, 'add_metadef_namespace', {})
         return super(MetadefNamespaceRepoProxy, self).add(namespace)
 
+    def remove(self, namespace):
+        self.policy.enforce(self.context, 'delete_metadef_namespace', {})
+        return super(MetadefNamespaceRepoProxy, self).remove(namespace)
+
+    def remove_tags(self, namespace):
+        self.policy.enforce(self.context, 'delete_metadef_tags', {})
+        return super(MetadefNamespaceRepoProxy, self).remove_tags(namespace)
+
 
 class MetadefNamespaceFactoryProxy(
         glance.domain.proxy.MetadefNamespaceFactory):
@@ -468,6 +513,10 @@ class MetadefObjectProxy(glance.domain.proxy.MetadefObject):
         self.policy = policy
         super(MetadefObjectProxy, self).__init__(meta_object)
 
+    def delete(self):
+        self.policy.enforce(self.context, 'delete_metadef_object', {})
+        return super(MetadefObjectProxy, self).delete()
+
 
 class MetadefObjectRepoProxy(glance.domain.proxy.MetadefObjectRepo):
 
@@ -497,6 +546,10 @@ class MetadefObjectRepoProxy(glance.domain.proxy.MetadefObjectRepo):
         self.policy.enforce(self.context, 'add_metadef_object', {})
         return super(MetadefObjectRepoProxy, self).add(meta_object)
 
+    def remove(self, meta_object):
+        self.policy.enforce(self.context, 'delete_metadef_object', {})
+        return super(MetadefObjectRepoProxy, self).remove(meta_object)
+
 
 class MetadefObjectFactoryProxy(glance.domain.proxy.MetadefObjectFactory):
 
@@ -520,6 +573,11 @@ class MetadefResourceTypeProxy(glance.domain.proxy.MetadefResourceType):
         self.policy = policy
         super(MetadefResourceTypeProxy, self).__init__(meta_resource_type)
 
+    def delete(self):
+        self.policy.enforce(self.context,
+                            'remove_metadef_resource_type_association', {})
+        return super(MetadefResourceTypeProxy, self).delete()
+
 
 class MetadefResourceTypeRepoProxy(
         glance.domain.proxy.MetadefResourceTypeRepo):
@@ -547,6 +605,12 @@ class MetadefResourceTypeRepoProxy(
                             'add_metadef_resource_type_association', {})
         return super(MetadefResourceTypeRepoProxy, self).add(resource_type)
 
+    def remove(self, *args, **kwargs):
+        self.policy.enforce(self.context,
+                            'remove_metadef_resource_type_association', {})
+        return super(MetadefResourceTypeRepoProxy,
+                     self).remove(*args, **kwargs)
+
 
 class MetadefResourceTypeFactoryProxy(
         glance.domain.proxy.MetadefResourceTypeFactory):
@@ -571,6 +635,10 @@ class MetadefPropertyProxy(glance.domain.proxy.MetadefProperty):
         self.policy = policy
         super(MetadefPropertyProxy, self).__init__(namespace_property)
 
+    def delete(self):
+        self.policy.enforce(self.context, 'remove_metadef_property', {})
+        return super(MetadefPropertyProxy, self).delete()
+
 
 class MetadefPropertyRepoProxy(glance.domain.proxy.MetadefPropertyRepo):
 
@@ -604,6 +672,10 @@ class MetadefPropertyRepoProxy(glance.domain.proxy.MetadefPropertyRepo):
         return super(MetadefPropertyRepoProxy, self).add(
             namespace_property)
 
+    def remove(self, *args, **kwargs):
+        self.policy.enforce(self.context, 'remove_metadef_property', {})
+        return super(MetadefPropertyRepoProxy, self).remove(*args, **kwargs)
+
 
 class MetadefPropertyFactoryProxy(glance.domain.proxy.MetadefPropertyFactory):
 
@@ -626,6 +698,10 @@ class MetadefTagProxy(glance.domain.proxy.MetadefTag):
         self.policy = policy
         super(MetadefTagProxy, self).__init__(meta_tag)
 
+    def delete(self):
+        self.policy.enforce(self.context, 'delete_metadef_tag', {})
+        return super(MetadefTagProxy, self).delete()
+
 
 class MetadefTagRepoProxy(glance.domain.proxy.MetadefTagRepo):
 
@@ -659,6 +735,10 @@ class MetadefTagRepoProxy(glance.domain.proxy.MetadefTagRepo):
         self.policy.enforce(self.context, 'add_metadef_tags', {})
         return super(MetadefTagRepoProxy, self).add_tags(meta_tags)
 
+    def remove(self, meta_tag):
+        self.policy.enforce(self.context, 'delete_metadef_tag', {})
+        return super(MetadefTagRepoProxy, self).remove(meta_tag)
+
 
 class MetadefTagFactoryProxy(glance.domain.proxy.MetadefTagFactory):
 
