@@ -17,6 +17,7 @@ from oslo_config import cfg
 from oslo_log import log as logging
 from oslo_serialization import jsonutils
 from six.moves import http_client
+from six.moves import urllib
 import webob.dec
 
 from glance.common import wsgi
@@ -59,34 +60,43 @@ class Controller(object):
     def index(self, req, explicit=False):
         """Respond to a request for all OpenStack API versions."""
         def build_version_object(version, path, status):
-            url = CONF.public_endpoint or req.host_url
+            url = CONF.public_endpoint or req.application_url
+            # Always add '/' to url end for urljoin href url
+            url = url.rstrip('/') + '/'
+            href = urllib.parse.urljoin(url, path).rstrip('/') + '/'
             return {
                 'id': 'v%s' % version,
                 'status': status,
                 'links': [
                     {
                         'rel': 'self',
-                        'href': '%s/%s/' % (url, path),
+                        'href': '%s' % href,
                     },
                 ],
             }
 
         version_objs = []
-        if CONF.enable_v2_api:
-            if CONF.enabled_backends:
-                version_objs.extend([
-                    build_version_object(2.8, 'v2', 'EXPERIMENTAL')
-                ])
+        if CONF.enabled_backends:
             version_objs.extend([
-                build_version_object(2.7, 'v2', 'CURRENT'),
-                build_version_object(2.6, 'v2', 'SUPPORTED'),
-                build_version_object(2.5, 'v2', 'SUPPORTED'),
-                build_version_object(2.4, 'v2', 'SUPPORTED'),
-                build_version_object(2.3, 'v2', 'SUPPORTED'),
-                build_version_object(2.2, 'v2', 'SUPPORTED'),
-                build_version_object(2.1, 'v2', 'SUPPORTED'),
-                build_version_object(2.0, 'v2', 'SUPPORTED'),
+                build_version_object(2.11, 'v2', 'CURRENT'),
+                build_version_object('2.10', 'v2', 'SUPPORTED'),
+                build_version_object(2.9, 'v2', 'SUPPORTED'),
+                build_version_object(2.8, 'v2', 'SUPPORTED'),
             ])
+        else:
+            version_objs.extend([
+                build_version_object(2.9, 'v2', 'CURRENT'),
+            ])
+        version_objs.extend([
+            build_version_object(2.7, 'v2', 'SUPPORTED'),
+            build_version_object(2.6, 'v2', 'SUPPORTED'),
+            build_version_object(2.5, 'v2', 'SUPPORTED'),
+            build_version_object(2.4, 'v2', 'SUPPORTED'),
+            build_version_object(2.3, 'v2', 'SUPPORTED'),
+            build_version_object(2.2, 'v2', 'SUPPORTED'),
+            build_version_object(2.1, 'v2', 'SUPPORTED'),
+            build_version_object(2.0, 'v2', 'SUPPORTED'),
+        ])
 
         status = explicit and http_client.OK or http_client.MULTIPLE_CHOICES
         response = webob.Response(request=req,
