@@ -16,8 +16,6 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-from __future__ import print_function
-
 import os
 import sys
 
@@ -27,7 +25,7 @@ from oslo_serialization import jsonutils
 from oslo_utils import encodeutils
 from oslo_utils import uuidutils
 import six
-from six.moves import http_client
+from six.moves import http_client as http
 import six.moves.urllib.parse as urlparse
 from webob import exc
 
@@ -59,28 +57,33 @@ cli_opts = [
                default='',
                help=("Pass in your authentication token if you have "
                      "one. If you use this option the same token is "
-                     "used for both the master and the slave.")),
-    cfg.StrOpt('mastertoken',
-               short='M',
-               default='',
-               help=("Pass in your authentication token if you have "
-                     "one. This is the token used for the master.")),
-    cfg.StrOpt('slavetoken',
-               short='S',
-               default='',
-               help=("Pass in your authentication token if you have "
-                     "one. This is the token used for the slave.")),
+                     "used for both the source and the target.")),
     cfg.StrOpt('command',
                positional=True,
+               required=False,
                help="Command to be given to replicator"),
     cfg.MultiStrOpt('args',
                     positional=True,
+                    required=False,
                     help="Arguments for the command"),
 ]
 
 CONF = cfg.CONF
 CONF.register_cli_opts(cli_opts)
+
+CONF.register_opt(
+    cfg.StrOpt('sourcetoken',
+               default='',
+               help=("Pass in your authentication token if you have "
+                     "one. This is the token used for the source.")))
+CONF.register_opt(
+    cfg.StrOpt('targettoken',
+               default='',
+               help=("Pass in your authentication token if you have "
+                     "one. This is the token used for the target.")))
+
 logging.register_options(CONF)
+CONF.set_default(name='use_stderr', default=True)
 
 # If ../glance/__init__.py exists, add ../ to Python search path, so that
 # it will override what happens to be installed in /usr/(local/)lib/python...
@@ -95,7 +98,7 @@ COMMANDS = """Commands:
 
     help <command>  Output help for one of the commands below
 
-    compare         What is missing from the slave glance?
+    compare         What is missing from the target glance?
     dump            Dump the contents of a glance instance to local disk.
     livecopy        Load the contents of one glance instance into another.
     load            Load the contents of a local directory into glance.
@@ -104,10 +107,10 @@ COMMANDS = """Commands:
 
 
 IMAGE_ALREADY_PRESENT_MESSAGE = _('The image %s is already present on '
-                                  'the slave, but our check for it did '
+                                  'the target, but our check for it did '
                                   'not find it. This indicates that we '
                                   'do not have permissions to see all '
-                                  'the images on the slave server.')
+                                  'the images on the target server.')
 
 
 class ImageService(object):
@@ -147,29 +150,29 @@ class ImageService(object):
         response = self.conn.getresponse()
         headers = self._header_list_to_dict(response.getheaders())
         code = response.status
-        code_description = http_client.responses[code]
+        code_description = http.responses[code]
         LOG.debug('Response: %(code)s %(status)s %(headers)s',
                   {'code': code,
                    'status': code_description,
                    'headers': repr(headers)})
 
-        if code == 400:
+        if code == http.BAD_REQUEST:
             raise exc.HTTPBadRequest(
                 explanation=response.read())
 
-        if code == 500:
+        if code == http.INTERNAL_SERVER_ERROR:
             raise exc.HTTPInternalServerError(
                 explanation=response.read())
 
-        if code == 401:
+        if code == http.UNAUTHORIZED:
             raise exc.HTTPUnauthorized(
                 explanation=response.read())
 
-        if code == 403:
+        if code == http.FORBIDDEN:
             raise exc.HTTPForbidden(
                 explanation=response.read())
 
-        if code == 409:
+        if code == http.CONFLICT:
             raise exc.HTTPConflict(
                 explanation=response.read())
 
@@ -339,8 +342,8 @@ def replication_size(options, args):
     count = 0
 
     imageservice = get_image_service()
-    client = imageservice(http_client.HTTPConnection(server, port),
-                          options.slavetoken)
+    client = imageservice(http.HTTPConnection(server, port),
+                          options.targettoken)
     for image in client.get_images():
         LOG.debug('Considering image: %(image)s', {'image': image})
         if image['status'] == 'active':
@@ -371,8 +374,8 @@ def replication_dump(options, args):
     server, port = utils.parse_valid_host_port(args.pop())
 
     imageservice = get_image_service()
-    client = imageservice(http_client.HTTPConnection(server, port),
-                          options.mastertoken)
+    client = imageservice(http.HTTPConnection(server, port),
+                          options.sourcetoken)
     for image in client.get_images():
         LOG.debug('Considering: %(image_id)s (%(image_name)s) '
                   '(%(image_size)d bytes)',
@@ -421,20 +424,20 @@ def _dict_diff(a, b):
 
     Returns: True if the dictionaries are different
     """
-    # Only things the master has which the slave lacks matter
+    # Only things the source has which the target lacks matter
     if set(a.keys()) - set(b.keys()):
-        LOG.debug('metadata diff -- master has extra keys: %(keys)s',
+        LOG.debug('metadata diff -- source has extra keys: %(keys)s',
                   {'keys': ' '.join(set(a.keys()) - set(b.keys()))})
         return True
 
     for key in a:
         if str(a[key]) != str(b[key]):
             LOG.debug('metadata diff -- value differs for key '
-                      '%(key)s: master "%(master_value)s" vs '
-                      'slave "%(slave_value)s"',
+                      '%(key)s: source "%(source_value)s" vs '
+                      'target "%(target_value)s"',
                       {'key': key,
-                       'master_value': a[key],
-                       'slave_value': b[key]})
+                       'source_value': a[key],
+                       'target_value': b[key]})
             return True
 
     return False
@@ -457,8 +460,8 @@ def replication_load(options, args):
     server, port = utils.parse_valid_host_port(args.pop())
 
     imageservice = get_image_service()
-    client = imageservice(http_client.HTTPConnection(server, port),
-                          options.slavetoken)
+    client = imageservice(http.HTTPConnection(server, port),
+                          options.targettoken)
 
     updated = []
 
@@ -486,7 +489,7 @@ def replication_load(options, args):
                 headers = client.get_image_meta(image_uuid)
                 for key in options.dontreplicate.split(' '):
                     if key in headers:
-                        LOG.debug('Stripping %(header)s from slave '
+                        LOG.debug('Stripping %(header)s from target '
                                   'metadata', {'header': key})
                         del headers[key]
 
@@ -520,8 +523,8 @@ def replication_livecopy(options, args):
 
     Load the contents of one glance instance into another.
 
-    fromserver:port: the location of the master glance instance.
-    toserver:port:   the location of the slave glance instance.
+    fromserver:port: the location of the source glance instance.
+    toserver:port:   the location of the target glance instance.
     """
 
     # Make sure from-server and to-server are provided
@@ -530,37 +533,37 @@ def replication_livecopy(options, args):
 
     imageservice = get_image_service()
 
-    slave_server, slave_port = utils.parse_valid_host_port(args.pop())
-    slave_conn = http_client.HTTPConnection(slave_server, slave_port)
-    slave_client = imageservice(slave_conn, options.slavetoken)
+    target_server, target_port = utils.parse_valid_host_port(args.pop())
+    target_conn = http.HTTPConnection(target_server, target_port)
+    target_client = imageservice(target_conn, options.targettoken)
 
-    master_server, master_port = utils.parse_valid_host_port(args.pop())
-    master_conn = http_client.HTTPConnection(master_server, master_port)
-    master_client = imageservice(master_conn, options.mastertoken)
+    source_server, source_port = utils.parse_valid_host_port(args.pop())
+    source_conn = http.HTTPConnection(source_server, source_port)
+    source_client = imageservice(source_conn, options.sourcetoken)
 
     updated = []
 
-    for image in master_client.get_images():
+    for image in source_client.get_images():
         LOG.debug('Considering %(id)s', {'id': image['id']})
         for key in options.dontreplicate.split(' '):
             if key in image:
-                LOG.debug('Stripping %(header)s from master metadata',
+                LOG.debug('Stripping %(header)s from source metadata',
                           {'header': key})
                 del image[key]
 
-        if _image_present(slave_client, image['id']):
+        if _image_present(target_client, image['id']):
             # NOTE(mikal): Perhaps we just need to update the metadata?
             # Note that we don't attempt to change an image file once it
             # has been uploaded.
-            headers = slave_client.get_image_meta(image['id'])
+            headers = target_client.get_image_meta(image['id'])
             if headers['status'] == 'active':
                 for key in options.dontreplicate.split(' '):
                     if key in image:
-                        LOG.debug('Stripping %(header)s from master '
+                        LOG.debug('Stripping %(header)s from source '
                                   'metadata', {'header': key})
                         del image[key]
                     if key in headers:
-                        LOG.debug('Stripping %(header)s from slave '
+                        LOG.debug('Stripping %(header)s from target '
                                   'metadata', {'header': key})
                         del headers[key]
 
@@ -569,7 +572,7 @@ def replication_livecopy(options, args):
                                  'metadata has changed'),
                              {'image_id': image['id'],
                               'image_name': image.get('name', '--unnamed--')})
-                    headers, body = slave_client.add_image_meta(image)
+                    headers, body = target_client.add_image_meta(image)
                     _check_upload_response_headers(headers, body)
                     updated.append(image['id'])
 
@@ -581,10 +584,10 @@ def replication_livecopy(options, args):
                       'image_name': image.get('name', '--unnamed--'),
                       'image_size': image['size']})
             if not options.metaonly:
-                image_response = master_client.get_image(image['id'])
+                image_response = source_client.get_image(image['id'])
                 try:
-                    headers, body = slave_client.add_image(image,
-                                                           image_response)
+                    headers, body = target_client.add_image(image,
+                                                            image_response)
                     _check_upload_response_headers(headers, body)
                     updated.append(image['id'])
                 except exc.HTTPConflict:
@@ -598,8 +601,8 @@ def replication_compare(options, args):
 
     Compare the contents of fromserver with those of toserver.
 
-    fromserver:port: the location of the master glance instance.
-    toserver:port:   the location of the slave glance instance.
+    fromserver:port: the location of the source glance instance.
+    toserver:port:   the location of the target glance instance.
     """
 
     # Make sure from-server and to-server are provided
@@ -608,38 +611,39 @@ def replication_compare(options, args):
 
     imageservice = get_image_service()
 
-    slave_server, slave_port = utils.parse_valid_host_port(args.pop())
-    slave_conn = http_client.HTTPConnection(slave_server, slave_port)
-    slave_client = imageservice(slave_conn, options.slavetoken)
+    target_server, target_port = utils.parse_valid_host_port(args.pop())
+    target_conn = http.HTTPConnection(target_server, target_port)
+    target_client = imageservice(target_conn, options.targettoken)
 
-    master_server, master_port = utils.parse_valid_host_port(args.pop())
-    master_conn = http_client.HTTPConnection(master_server, master_port)
-    master_client = imageservice(master_conn, options.mastertoken)
+    source_server, source_port = utils.parse_valid_host_port(args.pop())
+    source_conn = http.HTTPConnection(source_server, source_port)
+    source_client = imageservice(source_conn, options.sourcetoken)
 
     differences = {}
 
-    for image in master_client.get_images():
-        if _image_present(slave_client, image['id']):
-            headers = slave_client.get_image_meta(image['id'])
+    for image in source_client.get_images():
+        if _image_present(target_client, image['id']):
+            headers = target_client.get_image_meta(image['id'])
             for key in options.dontreplicate.split(' '):
                 if key in image:
-                    LOG.debug('Stripping %(header)s from master metadata',
+                    LOG.debug('Stripping %(header)s from source metadata',
                               {'header': key})
                     del image[key]
                 if key in headers:
-                    LOG.debug('Stripping %(header)s from slave metadata',
+                    LOG.debug('Stripping %(header)s from target metadata',
                               {'header': key})
                     del headers[key]
 
             for key in image:
-                if image[key] != headers.get(key, None):
+                if image[key] != headers.get(key):
                     LOG.warn(_LW('%(image_id)s: field %(key)s differs '
-                                 '(source is %(master_value)s, destination '
-                                 'is %(slave_value)s)')
+                                 '(source is %(source_value)s, destination '
+                                 'is %(target_value)s)')
                              % {'image_id': image['id'],
                                 'key': key,
-                                'master_value': image[key],
-                                'slave_value': headers.get(key, 'undefined')})
+                                'source_value': image[key],
+                                'target_value': headers.get(key,
+                                                            'undefined')})
                     differences[image['id']] = 'diff'
                 else:
                     LOG.debug('%(image_id)s is identical',
@@ -734,15 +738,15 @@ def main():
         config.parse_args()
     except RuntimeError as e:
         sys.exit("ERROR: %s" % encodeutils.exception_to_unicode(e))
-    except SystemExit as e:
+    except SystemExit:
         sys.exit("Please specify one command")
 
     # Setup logging
     logging.setup(CONF, 'glance')
 
     if CONF.token:
-        CONF.slavetoken = CONF.token
-        CONF.mastertoken = CONF.token
+        CONF.sourcetoken = CONF.token
+        CONF.targettoken = CONF.token
 
     command = lookup_command(CONF.command)
 
