@@ -19,6 +19,7 @@ Routines for configuring Glance
 
 import logging
 import os
+import sys
 
 from oslo_config import cfg
 from oslo_middleware import cors
@@ -85,7 +86,8 @@ Related Options:
 ]
 image_format_opts = [
     cfg.ListOpt('container_formats',
-                default=['ami', 'ari', 'aki', 'bare', 'ovf', 'ova', 'docker'],
+                default=['ami', 'ari', 'aki', 'bare', 'ovf', 'ova', 'docker',
+                         'compressed'],
                 help=_("Supported values for the 'container_format' "
                        "image attribute"),
                 deprecated_opts=[cfg.DeprecatedOpt('container_formats',
@@ -158,18 +160,15 @@ Related Options:
 """)),
 ]
 
-_DEPRECATE_GLANCE_V1_MSG = _('The Images (Glance) version 1 API has been '
-                             'DEPRECATED in the Newton release and will be '
-                             'removed on or after Pike release, following '
-                             'the standard OpenStack deprecation policy. '
-                             'Hence, the configuration options specific to '
-                             'the Images (Glance) v1 API are hereby '
-                             'deprecated and subject to removal. Operators '
-                             'are advised to deploy the Images (Glance) v2 '
-                             'API.')
-
 common_opts = [
     cfg.BoolOpt('allow_additional_image_properties', default=True,
+                deprecated_for_removal=True,
+                deprecated_since="Ussuri",
+                deprecated_reason=_("""
+This option is redundant.  Control custom image property usage via the
+'image_property_quota' configuration option.  This option is scheduled
+to be removed during the Victoria development cycle.
+"""),
                 help=_("""
 Allow users to add additional/custom properties to images.
 
@@ -194,7 +193,7 @@ Related options:
 """)),
     cfg.StrOpt('hashing_algorithm',
                default='sha512',
-               help=_(""""
+               help=_("""
 Secure hashing algorithm used for computing the 'os_hash_value' property.
 
 This option configures the Glance "multihash", which consists of two
@@ -270,59 +269,6 @@ Any negative value is interpreted as unlimited.
 Related options:
     * None
 
-""")),
-    # TODO(abashmak): Add choices parameter to this option:
-    # choices('glance.db.sqlalchemy.api',
-    #         'glance.db.registry.api',
-    #         'glance.db.simple.api')
-    # This will require a fix to the functional tests which
-    # set this option to a test version of the registry api module:
-    # (glance.tests.functional.v2.registry_data_api), in order to
-    # bypass keystone authentication for the Registry service.
-    # All such tests are contained in:
-    # glance/tests/functional/v2/test_images.py
-    cfg.StrOpt('data_api',
-               default='glance.db.sqlalchemy.api',
-                       deprecated_for_removal=True,
-                       deprecated_since="Queens",
-                       deprecated_reason=_("""
-Glance registry service is deprecated for removal.
-
-More information can be found from the spec:
-http://specs.openstack.org/openstack/glance-specs/specs/queens/approved/glance/deprecate-registry.html
-"""),
-               help=_("""
-Python module path of data access API.
-
-Specifies the path to the API to use for accessing the data model.
-This option determines how the image catalog data will be accessed.
-
-Possible values:
-    * glance.db.sqlalchemy.api
-    * glance.db.registry.api
-    * glance.db.simple.api
-
-If this option is set to ``glance.db.sqlalchemy.api`` then the image
-catalog data is stored in and read from the database via the
-SQLAlchemy Core and ORM APIs.
-
-Setting this option to ``glance.db.registry.api`` will force all
-database access requests to be routed through the Registry service.
-This avoids data access from the Glance API nodes for an added layer
-of security, scalability and manageability.
-
-NOTE: In v2 OpenStack Images API, the registry service is optional.
-In order to use the Registry API in v2, the option
-``enable_v2_registry`` must be set to ``True``.
-
-Finally, when this configuration option is set to
-``glance.db.simple.api``, image catalog data is stored in and read
-from an in-memory data structure. This is primarily used for testing.
-
-Related options:
-    * enable_v2_api
-    * enable_v2_registry
-
 """)),
     cfg.IntOpt('limit_param_default', default=25, min=1,
                help=_("""
@@ -405,14 +351,26 @@ Related options:
 """)),
     # NOTE(flaper87): The policy.json file should be updated and the locaiton
     # related rules set to admin only once this option is finally removed.
+    # NOTE(rosmaita): Unfortunately, this option is used to gate some code
+    # paths; if the location related policies are set admin-only, then no
+    # normal users can save or retrieve image data.
     cfg.BoolOpt('show_multiple_locations', default=False,
                 deprecated_for_removal=True,
-                deprecated_reason=_('This option will be removed in the Pike '
-                                    'release or later because the same '
-                                    'functionality can be achieved with '
-                                    'greater granularity by using policies. '
-                                    'Please see the Newton '
-                                    'release notes for more information.'),
+                deprecated_reason=_('Use of this option, deprecated since '
+                                    'Newton, is a security risk and will be '
+                                    'removed once we figure out a way to '
+                                    'satisfy those use cases that currently '
+                                    'require it.  An earlier announcement '
+                                    'that the same functionality can be '
+                                    'achieved with greater granularity by '
+                                    'using policies is incorrect.  You cannot '
+                                    'work around this option via policy '
+                                    'configuration at the present time, '
+                                    'though that is the direction we believe '
+                                    'the fix will take.  Please keep an eye '
+                                    'on the Glance release notes to stay up '
+                                    'to date on progress in addressing this '
+                                    'issue.'),
                 deprecated_since='Newton',
                 help=_("""
 Show all image locations when returning an image.
@@ -429,6 +387,8 @@ NOTES:
       image locations can sometimes include credentials. Hence, this
       is set to ``False`` by default. Set this to ``True`` with
       EXTREME CAUTION and ONLY IF you know what you are doing!
+    * See https://wiki.openstack.org/wiki/OSSN/OSSN-0065 for more
+      information.
     * If an operator wishes to avoid showing any image location(s)
       to the user, then both this option and
       ``show_image_direct_url`` MUST be set to ``False``.
@@ -488,79 +448,6 @@ Possible values:
 Related options:
     * None
 
-""")),
-    cfg.BoolOpt('enable_v2_api',
-                default=True,
-                deprecated_reason=_('The Images (Glance) version 1 API has '
-                                    'been DEPRECATED in the Newton release. '
-                                    'It will be removed on or after Pike '
-                                    'release, following the standard '
-                                    'OpenStack deprecation policy. Once we '
-                                    'remove the Images (Glance) v1 API, only '
-                                    'the Images (Glance) v2 API can be '
-                                    'deployed and will be enabled by default '
-                                    'making this option redundant.'),
-                deprecated_since='Newton',
-                help=_("""
-Deploy the v2 OpenStack Images API.
-
-When this option is set to ``True``, Glance service will respond
-to requests on registered endpoints conforming to the v2 OpenStack
-Images API.
-
-NOTES:
-    * If this option is disabled, then the ``enable_v2_registry``
-      option, which is enabled by default, is also recommended
-      to be disabled.
-
-Possible values:
-    * True
-    * False
-
-Related options:
-    * enable_v2_registry
-
-""")),
-    cfg.BoolOpt('enable_v1_registry',
-                default=True,
-                deprecated_reason=_DEPRECATE_GLANCE_V1_MSG,
-                deprecated_since='Newton',
-                help=_("""
-                    DEPRECATED FOR REMOVAL
-""")),
-    cfg.BoolOpt('enable_v2_registry',
-                default=True,
-                deprecated_for_removal=True,
-                deprecated_since="Queens",
-                deprecated_reason=_("""
-Glance registry service is deprecated for removal.
-
-More information can be found from the spec:
-http://specs.openstack.org/openstack/glance-specs/specs/queens/approved/glance/deprecate-registry.html
-"""),
-                help=_("""
-Deploy the v2 API Registry service.
-
-When this option is set to ``True``, the Registry service
-will be enabled in Glance for v2 API requests.
-
-NOTES:
-    * Use of Registry is optional in v2 API, so this option
-      must only be enabled if both ``enable_v2_api`` is set to
-      ``True`` and the ``data_api`` option is set to
-      ``glance.db.registry.api``.
-
-    * If deploying only the v1 OpenStack Images API, this option,
-      which is enabled by default, should be disabled.
-
-Possible values:
-    * True
-    * False
-
-Related options:
-    * enable_v2_api
-    * data_api
-
 """)),
     cfg.HostAddressOpt('pydev_worker_debug_host',
                        sample_default='localhost',
@@ -669,21 +556,46 @@ Related options:
     cfg.ListOpt('enabled_import_methods',
                 item_type=cfg.types.String(quotes=True),
                 bounds=True,
-                default=['glance-direct', 'web-download'],
+                default=['glance-direct', 'web-download',
+                         'copy-image'],
                 help=_("""
-List of enabled Image Import Methods
+    List of enabled Image Import Methods
 
-Both 'glance-direct' and 'web-download' are enabled by default.
+    'glance-direct', 'copy-image' and 'web-download' are enabled by default.
 
-Related options:
-    * [DEFAULT]/node_staging_uri""")),
+    Related options:
+        * [DEFAULT]/node_staging_uri""")),
 ]
 
+wsgi_opts = [
+    cfg.IntOpt('task_pool_threads',
+               default=16,
+               min=1,
+               help=_("""
+The number of threads (per worker process) in the pool for processing
+asynchronous tasks. This controls how many asynchronous tasks (i.e. for
+image interoperable import) each worker can run at a time. If this is
+too large, you *may* have increased memory footprint per worker and/or you
+may overwhelm other system resources such as disk or outbound network
+bandwidth. If this is too small, image import requests will have to wait
+until a thread becomes available to begin processing.""")),
+    cfg.StrOpt('python_interpreter',
+               default=sys.executable,
+               help=_("""
+Path to the python interpreter to use when spawning external
+processes. By default this is sys.executable, which should be the
+same interpreter running Glance itself. However, in some situations
+(i.e. uwsgi) this may not actually point to a python interpreter
+itself.""")),
+]
+
+
 CONF = cfg.CONF
 CONF.register_opts(paste_deploy_opts, group='paste_deploy')
 CONF.register_opts(image_format_opts, group='image_format')
 CONF.register_opts(task_opts, group='task')
 CONF.register_opts(common_opts)
+CONF.register_opts(wsgi_opts, group='wsgi')
 policy.Enforcer(CONF)
 
 
