@@ -234,11 +234,11 @@ def _populate_metadata(meta, metadata_path=None, merge=False,
             continue
 
         values = {
-            'namespace': metadata.get('namespace', None),
-            'display_name': metadata.get('display_name', None),
-            'description': metadata.get('description', None),
-            'visibility': metadata.get('visibility', None),
-            'protected': metadata.get('protected', None),
+            'namespace': metadata.get('namespace'),
+            'display_name': metadata.get('display_name'),
+            'description': metadata.get('description'),
+            'visibility': metadata.get('visibility'),
+            'protected': metadata.get('protected'),
             'owner': metadata.get('owner', 'admin')
         }
 
@@ -296,8 +296,8 @@ def _populate_metadata(meta, metadata_path=None, merge=False,
                 'namespace_id': namespace_id,
                 'resource_type_id': rt_id,
                 'properties_target': resource_type.get(
-                    'properties_target', None),
-                'prefix': resource_type.get('prefix', None)
+                    'properties_target'),
+                'prefix': resource_type.get('prefix')
             }
             namespace_resource_type = _get_namespace_resource_type_by_ids(
                 meta, namespace_id, rt_id)
@@ -329,10 +329,10 @@ def _populate_metadata(meta, metadata_path=None, merge=False,
         for object in metadata.get('objects', []):
             values = {
                 'name': object['name'],
-                'description': object.get('description', None),
+                'description': object.get('description'),
                 'namespace_id': namespace_id,
                 'json_schema': json.dumps(
-                    object.get('properties', None))
+                    object.get('properties'))
             }
             object_id = _get_resource_id(objects_table, namespace_id,
                                          object['name'])
@@ -395,7 +395,7 @@ def _export_data_to_file(meta, path):
     namespace_table = get_metadef_namespaces_table(meta)
     namespaces = namespace_table.select().execute().fetchall()
 
-    pattern = re.compile('[\W_]+', re.UNICODE)
+    pattern = re.compile(r'[\W_]+', re.UNICODE)
 
     for id, namespace in enumerate(namespaces, start=1):
         namespace_id = namespace['id']
@@ -464,6 +464,8 @@ def _export_data_to_file(meta, path):
 
         try:
             file_name = ''.join([path, namespace_file_name, '.json'])
+            if isfile(file_name):
+                LOG.info(_LI("Overwriting: %s"), file_name)
             with open(file_name, 'w') as json_file:
                 json_file.write(json.dumps(values))
         except Exception as e:
