@@ -14,7 +14,7 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import collections
+from collections import abc
 import datetime
 import uuid
 
@@ -283,11 +283,11 @@ class Image(object):
     def get_data(self, *args, **kwargs):
         raise NotImplementedError()
 
-    def set_data(self, data, size=None, backend=None):
+    def set_data(self, data, size=None, backend=None, set_active=True):
         raise NotImplementedError()
 
 
-class ExtraProperties(collections.MutableMapping, dict):
+class ExtraProperties(abc.MutableMapping, dict):
 
     def __getitem__(self, key):
         return dict.__getitem__(self, key)
@@ -300,9 +300,9 @@ class ExtraProperties(collections.MutableMapping, dict):
 
     def __eq__(self, other):
         if isinstance(other, ExtraProperties):
-            return dict(self).__eq__(dict(other))
+            return dict.__eq__(self, dict(other))
         elif isinstance(other, dict):
-            return dict(self).__eq__(other)
+            return dict.__eq__(self, other)
         else:
             return False
 
@@ -310,10 +310,10 @@ class ExtraProperties(collections.MutableMapping, dict):
         return not self.__eq__(other)
 
     def __len__(self):
-        return dict(self).__len__()
+        return dict.__len__(self)
 
     def keys(self):
-        return dict(self).keys()
+        return dict.keys(self)
 
 
 class ImageMembership(object):
@@ -360,10 +360,10 @@ class Task(object):
                  task_input, result, message):
 
         if task_type not in self._supported_task_type:
-            raise exception.InvalidTaskType(task_type)
+            raise exception.InvalidTaskType(type=task_type)
 
         if status not in self._supported_task_status:
-            raise exception.InvalidTaskStatus(status)
+            raise exception.InvalidTaskStatus(status=status)
 
         self.task_id = task_id
         self._status = status
@@ -491,10 +491,11 @@ class TaskFactory(object):
 class TaskExecutorFactory(object):
     eventlet_deprecation_warned = False
 
-    def __init__(self, task_repo, image_repo, image_factory):
+    def __init__(self, task_repo, image_repo, image_factory, admin_repo=None):
         self.task_repo = task_repo
         self.image_repo = image_repo
         self.image_factory = image_factory
+        self.admin_repo = admin_repo
 
     def new_task_executor(self, context):
         try:
@@ -513,14 +514,15 @@ class TaskExecutorFactory(object):
                     TaskExecutorFactory.eventlet_deprecation_warned = True
                 task_executor = 'taskflow'
 
-            executor_cls = ('glance.async.%s_executor.'
+            executor_cls = ('glance.async_.%s_executor.'
                             'TaskExecutor' % task_executor)
             LOG.debug("Loading %s executor", task_executor)
             executor = importutils.import_class(executor_cls)
             return executor(context,
                             self.task_repo,
                             self.image_repo,
-                            self.image_factory)
+                            self.image_factory,
+                            admin_repo=self.admin_repo)
         except ImportError:
             with excutils.save_and_reraise_exception():
                 LOG.exception(_LE("Failed to load the %s executor provided "
