@@ -13,7 +13,7 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import collections
+from collections import abc
 import copy
 import functools
 
@@ -27,6 +27,7 @@ from oslo_utils import encodeutils
 from oslo_utils import excutils
 
 from glance.common import exception
+from glance.common import format_inspector
 from glance.common import utils
 import glance.domain.proxy
 from glance.i18n import _, _LE, _LI, _LW
@@ -61,7 +62,7 @@ class ImageRepoProxy(glance.domain.proxy.Repo):
         for location in image.locations:
             if CONF.enabled_backends:
                 self.store_api.set_acls_for_multi_store(
-                    location['url'], location['metadata']['backend'],
+                    location['url'], location['metadata'].get('store'),
                     public=public, read_tenants=member_ids,
                     context=self.context
                 )
@@ -121,7 +122,7 @@ def _check_location_uri(context, store_api, store_utils, uri,
 def _check_image_location(context, store_api, store_utils, location):
     backend = None
     if CONF.enabled_backends:
-        backend = location['metadata'].get('backend')
+        backend = location['metadata'].get('store')
 
     _check_location_uri(context, store_api, store_utils, location['url'],
                         backend=backend)
@@ -133,7 +134,7 @@ def _set_image_size(context, image, locations):
         for location in locations:
             if CONF.enabled_backends:
                 size_from_backend = store.get_size_from_uri_and_backend(
-                    location['url'], location['metadata'].get('backend'),
+                    location['url'], location['metadata'].get('store'),
                     context=context)
             else:
                 size_from_backend = store.get_size_from_backend(
@@ -186,7 +187,7 @@ class ImageFactoryProxy(glance.domain.proxy.ImageFactory):
 
 
 @functools.total_ordering
-class StoreLocations(collections.MutableSequence):
+class StoreLocations(abc.MutableSequence):
     """
     The proxy for store location property. It takes responsibility for::
 
@@ -238,6 +239,17 @@ class StoreLocations(collections.MutableSequence):
                 self.image_proxy.context,
                 self.image_proxy.image.image_id,
                 location)
+        except store.exceptions.NotFound:
+            # NOTE(rosmaita): This can happen if the data was deleted by an
+            # operator from the backend, or a race condition from multiple
+            # delete-from-store requests. The old way to deal with this was
+            # that the user could just delete the image when the data is gone,
+            # but with multi-store, that is no longer a good option. So we
+            # intentionally leave the location popped (in other words, the
+            # pop() succeeds) but we also reraise the NotFound so that the
+            # calling code knows what happened.
+            with excutils.save_and_reraise_exception():
+                pass
         except Exception:
             with excutils.save_and_reraise_exception():
                 self.value.insert(i, location)
@@ -430,31 +442,17 @@ class ImageProxy(glance.domain.proxy.Image):
                     self.image.image_id,
                     location)
 
-    def set_data(self, data, size=None, backend=None):
-        if size is None:
-            size = 0  # NOTE(markwash): zero -> unknown size
+    def _upload_to_store(self, data, verifier, store=None, size=None):
+        """
+        Upload data to store
 
-        # Create the verifier for signature verification (if correct properties
-        # are present)
-        extra_props = self.image.extra_properties
-        if (signature_utils.should_create_verifier(extra_props)):
-            # NOTE(bpoulos): if creating verifier fails, exception will be
-            # raised
-            img_signature = extra_props[signature_utils.SIGNATURE]
-            hash_method = extra_props[signature_utils.HASH_METHOD]
-            key_type = extra_props[signature_utils.KEY_TYPE]
-            cert_uuid = extra_props[signature_utils.CERT_UUID]
-            verifier = signature_utils.get_verifier(
-                context=self.context,
-                img_signature_certificate_uuid=cert_uuid,
-                img_signature_hash_method=hash_method,
-                img_signature=img_signature,
-                img_signature_key_type=key_type
-            )
-        else:
-            verifier = None
-
-        hashing_algo = CONF['hashing_algorithm']
+        :param data: data to upload to store
+        :param verifier: for signature verification
+        :param store: store to upload data to
+        :param size: data size
+        :return:
+        """
+        hashing_algo = self.image.os_hash_algo or CONF['hashing_algorithm']
         if CONF.enabled_backends:
             (location, size, checksum,
              multihash, loc_meta) = self.store_api.add_with_multihash(
@@ -463,7 +461,7 @@ class ImageProxy(glance.domain.proxy.Image):
                 utils.LimitingReader(utils.CooperativeReader(data),
                                      CONF.image_size_cap),
                 size,
-                backend,
+                store,
                 hashing_algo,
                 context=self.context,
                 verifier=verifier)
@@ -481,16 +479,33 @@ class ImageProxy(glance.domain.proxy.Image):
                 hashing_algo,
                 context=self.context,
                 verifier=verifier)
+        self._verify_signature(verifier, location, loc_meta)
+        for attr, data in {"size": size, "os_hash_value": multihash,
+                           "checksum": checksum}.items():
+            self._verify_uploaded_data(data, attr)
+        self.image.locations.append({'url': location, 'metadata': loc_meta,
+                                     'status': 'active'})
+        self.image.checksum = checksum
+        self.image.os_hash_value = multihash
+        self.image.size = size
+        self.image.os_hash_algo = hashing_algo
 
+    def _verify_signature(self, verifier, location, loc_meta):
+        """
+        Verify signature of uploaded data.
+
+        :param verifier: for signature verification
+        """
         # NOTE(bpoulos): if verification fails, exception will be raised
-        if verifier:
+        if verifier is not None:
             try:
                 verifier.verify()
-                LOG.info(_LI("Successfully verified signature for image %s"),
-                         self.image.image_id)
+                msg = _LI("Successfully verified signature for image %s")
+                LOG.info(msg, self.image.image_id)
             except crypto_exception.InvalidSignature:
                 if CONF.enabled_backends:
-                    self.store_api.delete(location, loc_meta.get('backend'),
+                    self.store_api.delete(location,
+                                          loc_meta.get('store'),
                                           context=self.context)
                 else:
                     self.store_api.delete_from_backend(location,
@@ -499,13 +514,69 @@ class ImageProxy(glance.domain.proxy.Image):
                     _('Signature verification failed')
                 )
 
-        self.image.locations = [{'url': location, 'metadata': loc_meta,
-                                 'status': 'active'}]
-        self.image.size = size
-        self.image.checksum = checksum
-        self.image.os_hash_value = multihash
-        self.image.os_hash_algo = hashing_algo
-        self.image.status = 'active'
+    def _verify_uploaded_data(self, value, attribute_name):
+        """
+        Verify value of attribute_name uploaded data
+
+        :param value: value to compare
+        :param attribute_name: attribute name of the image to compare with
+        """
+        image_value = getattr(self.image, attribute_name)
+        if image_value is not None and value != image_value:
+            msg = _("%s of uploaded data is different from current "
+                    "value set on the image.")
+            LOG.error(msg, attribute_name)
+            raise exception.UploadException(msg % attribute_name)
+
+    def set_data(self, data, size=None, backend=None, set_active=True):
+        if size is None:
+            size = 0  # NOTE(markwash): zero -> unknown size
+
+        # Create the verifier for signature verification (if correct properties
+        # are present)
+        extra_props = self.image.extra_properties
+        verifier = None
+        if signature_utils.should_create_verifier(extra_props):
+            # NOTE(bpoulos): if creating verifier fails, exception will be
+            # raised
+            img_signature = extra_props[signature_utils.SIGNATURE]
+            hash_method = extra_props[signature_utils.HASH_METHOD]
+            key_type = extra_props[signature_utils.KEY_TYPE]
+            cert_uuid = extra_props[signature_utils.CERT_UUID]
+            verifier = signature_utils.get_verifier(
+                context=self.context,
+                img_signature_certificate_uuid=cert_uuid,
+                img_signature_hash_method=hash_method,
+                img_signature=img_signature,
+                img_signature_key_type=key_type
+            )
+
+        if not self.image.virtual_size:
+            inspector = format_inspector.get_inspector(self.image.disk_format)
+        else:
+            # No need to do this again
+            inspector = None
+
+        if inspector and self.image.container_format == 'bare':
+            fmt = inspector()
+            data = format_inspector.InfoWrapper(data, fmt)
+            LOG.debug('Enabling in-flight format inspection for %s', fmt)
+        else:
+            fmt = None
+
+        self._upload_to_store(data, verifier, backend, size)
+
+        if fmt and fmt.format_match and fmt.virtual_size:
+            self.image.virtual_size = fmt.virtual_size
+            LOG.info('Image format matched and virtual size computed: %i',
+                     self.image.virtual_size)
+        elif fmt:
+            LOG.warning('Image format %s did not match; '
+                        'unable to calculate virtual size',
+                        self.image.disk_format)
+
+        if set_active and self.image.status != 'active':
+            self.image.status = 'active'
 
     def get_data(self, offset=0, chunk_size=None):
         if not self.image.locations:
@@ -518,7 +589,7 @@ class ImageProxy(glance.domain.proxy.Image):
         err = None
         for loc in self.image.locations:
             try:
-                backend = loc['metadata'].get('backend')
+                backend = loc['metadata'].get('store')
                 if CONF.enabled_backends:
                     data, size = self.store_api.get(
                         loc['url'], backend, offset=offset,
@@ -534,14 +605,13 @@ class ImageProxy(glance.domain.proxy.Image):
                 return data
             except Exception as e:
                 LOG.warn(_LW('Get image %(id)s data failed: '
-                             '%(err)s.')
-                         % {'id': self.image.image_id,
-                            'err': encodeutils.exception_to_unicode(e)})
+                             '%(err)s.'),
+                         {'id': self.image.image_id,
+                          'err': encodeutils.exception_to_unicode(e)})
                 err = e
         # tried all locations
-        LOG.error(_LE(
-            'Glance tried all active locations/stores to get data '
-            'for image %s but all have failed.') % self.image.image_id)
+        LOG.error(_LE('Glance tried all active locations to get data for '
+                      'image %s but all have failed.'), self.image.image_id)
         raise err
 
 
@@ -558,9 +628,16 @@ class ImageMemberRepoProxy(glance.domain.proxy.Repo):
         if self.image.locations and not public:
             member_ids = [m.member_id for m in self.repo.list()]
             for location in self.image.locations:
-                self.store_api.set_acls(location['url'], public=public,
-                                        read_tenants=member_ids,
-                                        context=self.context)
+                if CONF.enabled_backends:
+                    self.store_api.set_acls_for_multi_store(
+                        location['url'], location['metadata'].get('store'),
+                        public=public, read_tenants=member_ids,
+                        context=self.context
+                    )
+                else:
+                    self.store_api.set_acls(location['url'], public=public,
+                                            read_tenants=member_ids,
+                                            context=self.context)
 
     def add(self, member):
         super(ImageMemberRepoProxy, self).add(member)
