@@ -14,10 +14,12 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import mock
-import os
 import tempfile
+from unittest import mock
 
+import glance_store as store
+from glance_store._drivers import cinder
+from oslo_config import cfg
 from oslo_log import log as logging
 import six
 import webob
@@ -25,34 +27,130 @@ import webob
 from glance.common import exception
 from glance.common import store_utils
 from glance.common import utils
+from glance.tests.unit import base
 from glance.tests import utils as test_utils
 
 
+CONF = cfg.CONF
+
+
 class TestStoreUtils(test_utils.BaseTestCase):
     """Test glance.common.store_utils module"""
 
-    def _test_update_store_in_location(self, metadata, store_id, expected):
+    def _test_update_store_in_location(self, metadata, store_id, expected,
+                                       store_id_call_count=1,
+                                       save_call_count=1):
+        image = mock.Mock()
+        image_repo = mock.Mock()
+        image_repo.save = mock.Mock()
+        context = mock.Mock()
         locations = [{
             'url': 'rbd://aaaaaaaa/images/id',
             'metadata': metadata
         }]
+        image.locations = locations
         with mock.patch.object(
                 store_utils, '_get_store_id_from_uri') as mock_get_store_id:
             mock_get_store_id.return_value = store_id
-            store_utils.update_store_in_locations(locations, mock.Mock())
-            self.assertEqual(locations[0]['metadata'].get('store'), expected)
+            store_utils.update_store_in_locations(context, image, image_repo)
+            self.assertEqual(image.locations[0]['metadata'].get(
+                'store'), expected)
+            self.assertEqual(store_id_call_count, mock_get_store_id.call_count)
+            self.assertEqual(save_call_count, image_repo.save.call_count)
 
     def test_update_store_location_with_no_store(self):
+        enabled_backends = {
+            "rbd1": "rbd",
+            "rbd2": "rbd"
+        }
+        self.config(enabled_backends=enabled_backends)
         self._test_update_store_in_location({}, 'rbd1', 'rbd1')
 
     def test_update_store_location_with_different_store(self):
-        self._test_update_store_in_location({'store': 'rbd2'}, 'rbd1', 'rbd1')
+        enabled_backends = {
+            "ceph1": "rbd",
+            "ceph2": "rbd"
+        }
+        self.config(enabled_backends=enabled_backends)
+        self._test_update_store_in_location(
+            {'store': 'rbd2'}, 'ceph1', 'ceph1')
 
     def test_update_store_location_with_same_store(self):
-        self._test_update_store_in_location({'store': 'rbd1'}, 'rbd1', 'rbd1')
+        enabled_backends = {
+            "rbd1": "rbd",
+            "rbd2": "rbd"
+        }
+        self.config(enabled_backends=enabled_backends)
+        self._test_update_store_in_location({'store': 'rbd1'}, 'rbd1', 'rbd1',
+                                            store_id_call_count=0,
+                                            save_call_count=0)
 
     def test_update_store_location_with_store_none(self):
-        self._test_update_store_in_location({}, None, None)
+        enabled_backends = {
+            "rbd1": "rbd",
+            "rbd2": "rbd"
+        }
+        self.config(enabled_backends=enabled_backends)
+        self._test_update_store_in_location({}, None, None,
+                                            save_call_count=0)
+
+
+class TestCinderStoreUtils(base.MultiStoreClearingUnitTest):
+    """Test glance.common.store_utils module for cinder multistore"""
+
+    @mock.patch.object(cinder.Store, 'is_image_associated_with_store')
+    @mock.patch.object(cinder.Store, 'url_prefix',
+                       new_callable=mock.PropertyMock)
+    def _test_update_cinder_store_in_location(self, mock_url_prefix,
+                                              mock_associate_store,
+                                              is_valid=True):
+        volume_id = 'db457a25-8f16-4b2c-a644-eae8d17fe224'
+        store_id = 'fast-cinder'
+        expected = 'fast-cinder'
+        image = mock.Mock()
+        image_repo = mock.Mock()
+        image_repo.save = mock.Mock()
+        context = mock.Mock()
+        mock_associate_store.return_value = is_valid
+        locations = [{
+            'url': 'cinder://%s' % volume_id,
+            'metadata': {}
+        }]
+        mock_url_prefix.return_value = 'cinder://%s' % store_id
+        image.locations = locations
+        store_utils.update_store_in_locations(context, image, image_repo)
+
+        if is_valid:
+            # This is the case where we found an image that has an
+            # old-style URL which does not include the store name,
+            # but for which we know the corresponding store that
+            # refers to the volume type that backs it. We expect that
+            # the URL should be updated to point to the store/volume from
+            # just a naked pointer to the volume, as was the old
+            # format i.e. this is the case when store is valid and location
+            # url, metadata are updated and image_repo.save is called
+            expected_url = mock_url_prefix.return_value + '/' + volume_id
+            self.assertEqual(expected_url, image.locations[0].get('url'))
+            self.assertEqual(expected, image.locations[0]['metadata'].get(
+                'store'))
+            self.assertEqual(1, image_repo.save.call_count)
+        else:
+            # Here, we've got an image backed by a volume which does
+            # not have a corresponding store specifying the volume_type.
+            # Expect that we leave these alone and do not touch the
+            # location URL since we cannot update it with a valid store i.e.
+            # this is the case when store is invalid and location url,
+            # metadata are not updated and image_repo.save is not called
+            self.assertEqual(locations[0]['url'],
+                             image.locations[0].get('url'))
+            self.assertEqual({}, image.locations[0]['metadata'])
+            self.assertEqual(0, image_repo.save.call_count)
+
+    def test_update_cinder_store_location_valid_type(self):
+        self._test_update_cinder_store_in_location()
+
+    def test_update_cinder_store_location_invalid_type(self):
+        self._test_update_cinder_store_in_location(is_valid=False)
 
 
 class TestUtils(test_utils.BaseTestCase):
@@ -326,46 +424,6 @@ class TestUtils(test_utils.BaseTestCase):
         result = utils.mutating(fake_function)
         self.assertEqual("test passed", result(req, Fake()))
 
-    def test_validate_key_cert_key(self):
-        self.config(digest_algorithm='sha256')
-        var_dir = os.path.abspath(os.path.join(os.path.dirname(__file__),
-                                               '../../', 'var'))
-        keyfile = os.path.join(var_dir, 'privatekey.key')
-        certfile = os.path.join(var_dir, 'certificate.crt')
-        utils.validate_key_cert(keyfile, certfile)
-
-    def test_validate_key_cert_no_private_key(self):
-        with tempfile.NamedTemporaryFile('w+') as tmpf:
-            self.assertRaises(RuntimeError,
-                              utils.validate_key_cert,
-                              "/not/a/file", tmpf.name)
-
-    def test_validate_key_cert_cert_cant_read(self):
-        with tempfile.NamedTemporaryFile('w+') as keyf:
-            with tempfile.NamedTemporaryFile('w+') as certf:
-                os.chmod(certf.name, 0)
-                self.assertRaises(RuntimeError,
-                                  utils.validate_key_cert,
-                                  keyf.name, certf.name)
-
-    def test_validate_key_cert_key_cant_read(self):
-        with tempfile.NamedTemporaryFile('w+') as keyf:
-            with tempfile.NamedTemporaryFile('w+') as certf:
-                os.chmod(keyf.name, 0)
-                self.assertRaises(RuntimeError,
-                                  utils.validate_key_cert,
-                                  keyf.name, certf.name)
-
-    def test_invalid_digest_algorithm(self):
-        self.config(digest_algorithm='fake_algorithm')
-        var_dir = os.path.abspath(os.path.join(os.path.dirname(__file__),
-                                               '../../', 'var'))
-        keyfile = os.path.join(var_dir, 'privatekey.key')
-        certfile = os.path.join(var_dir, 'certificate.crt')
-        self.assertRaises(ValueError,
-                          utils.validate_key_cert,
-                          keyfile, certfile)
-
     def test_valid_hostname(self):
         valid_inputs = ['localhost',
                         'glance04-a'
@@ -449,6 +507,148 @@ class TestUtils(test_utils.BaseTestCase):
                               utils.parse_valid_host_port,
                               pair)
 
+    def test_get_stores_from_request_returns_default(self):
+        enabled_backends = {
+            "ceph1": "rbd",
+            "ceph2": "rbd"
+        }
+        self.config(enabled_backends=enabled_backends)
+        store.register_store_opts(CONF)
+        self.config(default_backend="ceph1", group="glance_store")
+
+        req = webob.Request.blank('/some_request')
+        mp = "glance.common.utils.glance_store.get_store_from_store_identifier"
+        with mock.patch(mp) as mock_get_store:
+            result = utils.get_stores_from_request(req, {})
+            self.assertEqual(["ceph1"], result)
+            mock_get_store.assert_called_once_with("ceph1")
+
+    def test_get_stores_from_request_returns_stores_from_body(self):
+        enabled_backends = {
+            "ceph1": "rbd",
+            "ceph2": "rbd"
+        }
+        self.config(enabled_backends=enabled_backends)
+        store.register_store_opts(CONF)
+        self.config(default_backend="ceph1", group="glance_store")
+
+        body = {"stores": ["ceph1", "ceph2"]}
+        req = webob.Request.blank("/some_request")
+        mp = "glance.common.utils.glance_store.get_store_from_store_identifier"
+        with mock.patch(mp) as mock_get_store:
+            result = utils.get_stores_from_request(req, body)
+            self.assertEqual(["ceph1", "ceph2"], result)
+            mock_get_store.assert_any_call("ceph1")
+            mock_get_store.assert_any_call("ceph2")
+            self.assertEqual(mock_get_store.call_count, 2)
+
+    def test_get_stores_from_request_returns_store_from_headers(self):
+        enabled_backends = {
+            "ceph1": "rbd",
+            "ceph2": "rbd"
+        }
+        self.config(enabled_backends=enabled_backends)
+        store.register_store_opts(CONF)
+        self.config(default_backend="ceph1", group="glance_store")
+
+        headers = {"x-image-meta-store": "ceph2"}
+        req = webob.Request.blank("/some_request", headers=headers)
+        mp = "glance.common.utils.glance_store.get_store_from_store_identifier"
+        with mock.patch(mp) as mock_get_store:
+            result = utils.get_stores_from_request(req, {})
+            self.assertEqual(["ceph2"], result)
+            mock_get_store.assert_called_once_with("ceph2")
+
+    def test_get_stores_from_request_raises_bad_request(self):
+        enabled_backends = {
+            "ceph1": "rbd",
+            "ceph2": "rbd"
+        }
+        self.config(enabled_backends=enabled_backends)
+        store.register_store_opts(CONF)
+        self.config(default_backend="ceph1", group="glance_store")
+        headers = {"x-image-meta-store": "ceph2"}
+        body = {"stores": ["ceph1"]}
+        req = webob.Request.blank("/some_request", headers=headers)
+        self.assertRaises(webob.exc.HTTPBadRequest,
+                          utils.get_stores_from_request, req, body)
+
+    def test_get_stores_from_request_returns_all_stores(self):
+        enabled_backends = {
+            "ceph1": "rbd",
+            "ceph2": "rbd"
+        }
+        reserved_stores = {
+            'os_glance_staging_store': 'file',
+            'os_glance_tasks_store': 'file'
+        }
+        self.config(enabled_backends=enabled_backends)
+        store.register_store_opts(CONF, reserved_stores=reserved_stores)
+        self.config(default_backend="ceph1", group="glance_store")
+        body = {"all_stores": True}
+        req = webob.Request.blank("/some_request")
+        mp = "glance.common.utils.glance_store.get_store_from_store_identifier"
+        with mock.patch(mp) as mock_get_store:
+            result = sorted(utils.get_stores_from_request(req, body))
+            self.assertEqual(["ceph1", "ceph2"], result)
+            mock_get_store.assert_any_call("ceph1")
+            mock_get_store.assert_any_call("ceph2")
+            self.assertEqual(mock_get_store.call_count, 2)
+            self.assertNotIn('os_glance_staging_store', result)
+            self.assertNotIn('os_glance_tasks_store', result)
+
+    def test_get_stores_from_request_excludes_reserved_stores(self):
+        enabled_backends = {
+            "ceph1": "rbd",
+            "ceph2": "rbd"
+        }
+        self.config(enabled_backends=enabled_backends)
+        store.register_store_opts(CONF)
+        self.config(default_backend="ceph1", group="glance_store")
+        body = {"all_stores": True}
+        req = webob.Request.blank("/some_request")
+        mp = "glance.common.utils.glance_store.get_store_from_store_identifier"
+        with mock.patch(mp) as mock_get_store:
+            result = sorted(utils.get_stores_from_request(req, body))
+            self.assertEqual(["ceph1", "ceph2"], result)
+            mock_get_store.assert_any_call("ceph1")
+            mock_get_store.assert_any_call("ceph2")
+            self.assertEqual(mock_get_store.call_count, 2)
+
+    def test_get_stores_from_request_excludes_readonly_store(self):
+        enabled_backends = {
+            "ceph1": "rbd",
+            "ceph2": "rbd",
+            "http": "http"
+        }
+        self.config(enabled_backends=enabled_backends)
+        store.register_store_opts(CONF)
+        self.config(default_backend="ceph1", group="glance_store")
+        body = {"all_stores": True}
+        req = webob.Request.blank("/some_request")
+        mp = "glance.common.utils.glance_store.get_store_from_store_identifier"
+        with mock.patch(mp) as mock_get_store:
+            result = sorted(utils.get_stores_from_request(req, body))
+            self.assertNotIn("http", result)
+            self.assertEqual(["ceph1", "ceph2"], result)
+            mock_get_store.assert_any_call("ceph1")
+            mock_get_store.assert_any_call("ceph2")
+            self.assertEqual(mock_get_store.call_count, 2)
+
+    def test_get_stores_from_request_raises_bad_request_with_all_stores(self):
+        enabled_backends = {
+            "ceph1": "rbd",
+            "ceph2": "rbd"
+        }
+        self.config(enabled_backends=enabled_backends)
+        store.register_store_opts(CONF)
+        self.config(default_backend="ceph1", group="glance_store")
+        headers = {"x-image-meta-store": "ceph2"}
+        body = {"stores": ["ceph1"], "all_stores": True}
+        req = webob.Request.blank("/some_request", headers=headers)
+        self.assertRaises(webob.exc.HTTPBadRequest,
+                          utils.get_stores_from_request, req, body)
+
 
 class SplitFilterOpTestCase(test_utils.BaseTestCase):
 
