@@ -15,14 +15,14 @@
 #    under the License.
 
 import datetime
+from unittest import mock
 import uuid
 
-import mock
 from oslo_config import cfg
 import oslo_utils.importutils
 
-import glance.async
-from glance.async import taskflow_executor
+import glance.async_
+from glance.async_ import taskflow_executor
 from glance.common import exception
 from glance.common import timeutils
 from glance import domain
@@ -467,12 +467,12 @@ class TestTask(test_utils.BaseTestCase):
             self.task.expires_at
         )
 
-    @mock.patch.object(glance.async.TaskExecutor, 'begin_processing')
+    @mock.patch.object(glance.async_.TaskExecutor, 'begin_processing')
     def test_run(self, mock_begin_processing):
-        executor = glance.async.TaskExecutor(context=mock.ANY,
-                                             task_repo=mock.ANY,
-                                             image_repo=mock.ANY,
-                                             image_factory=mock.ANY)
+        executor = glance.async_.TaskExecutor(context=mock.ANY,
+                                              task_repo=mock.ANY,
+                                              image_repo=mock.ANY,
+                                              image_factory=mock.ANY)
         self.task.run(executor)
 
         mock_begin_processing.assert_called_once_with(self.task.task_id)
@@ -546,7 +546,28 @@ class TestTaskExecutorFactory(test_utils.BaseTestCase):
         mock_executor.assert_called_once_with(context,
                                               self.task_repo,
                                               self.image_repo,
-                                              self.image_factory)
+                                              self.image_factory,
+                                              admin_repo=None)
+
+    def test_new_task_executor_with_admin(self):
+        admin_repo = mock.MagicMock()
+        task_executor_factory = domain.TaskExecutorFactory(
+            self.task_repo,
+            self.image_repo,
+            self.image_factory,
+            admin_repo=admin_repo)
+        context = mock.Mock()
+        with mock.patch.object(oslo_utils.importutils,
+                               'import_class') as mock_import_class:
+            mock_executor = mock.Mock()
+            mock_import_class.return_value = mock_executor
+            task_executor_factory.new_task_executor(context)
+
+        mock_executor.assert_called_once_with(context,
+                                              self.task_repo,
+                                              self.image_repo,
+                                              self.image_factory,
+                                              admin_repo=admin_repo)
 
     def test_new_task_executor_error(self):
         task_executor_factory = domain.TaskExecutorFactory(self.task_repo,
