@@ -12,11 +12,9 @@
 #    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 #    License for the specific language governing permissions and limitations
 #    under the License.
-
-from __future__ import absolute_import
+from unittest import mock
 
 import fixtures
-import mock
 from six.moves import StringIO
 
 from glance.cmd import manage
@@ -378,6 +376,9 @@ class TestManage(TestManageBase):
                       ' revision. But, current revisions are: test ',
                       exit.code)
 
+    @mock.patch(
+        'glance.db.sqlalchemy.alembic_migrations.data_migrations.'
+        'has_pending_migrations')
     @mock.patch(
         'glance.db.sqlalchemy.alembic_migrations.get_current_alembic_heads')
     @mock.patch(
@@ -386,13 +387,15 @@ class TestManage(TestManageBase):
     @mock.patch.object(manage.DbCommands, '_sync')
     def test_contract(self, mock_sync, mock_validate_engine,
                       mock_get_alembic_branch_head,
-                      mock_get_current_alembic_heads):
+                      mock_get_current_alembic_heads,
+                      mock_has_pending_migrations):
         engine = mock_validate_engine.return_value
         engine.engine.name = 'mysql'
         mock_get_current_alembic_heads.side_effect = ['pike_expand01',
                                                       'pike_contract01']
         mock_get_alembic_branch_head.side_effect = ['pike_contract01',
                                                     'pike_expand01']
+        mock_has_pending_migrations.return_value = False
         self.db.contract()
         mock_sync.assert_called_once_with(version='pike_contract01')
 
