@@ -13,11 +13,11 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from unittest import mock
+from unittest.mock import patch
 import uuid
 
 import glance_store
-import mock
-from mock import patch
 from oslo_config import cfg
 # NOTE(jokke): simplified transition to py3, behaves like py2 xrange
 from six.moves import range
@@ -178,12 +178,11 @@ class ImagePager(object):
             page_size = image_count
         self.image_batches = []
         start = 0
-        l = len(images)
-        while start < l:
+        while start < image_count:
             self.image_batches.append(images[start: start + page_size])
             start += page_size
-            if (l - start) < page_size:
-                page_size = l - start
+            if (image_count - start) < page_size:
+                page_size = image_count - start
 
     def __call__(self):
         if len(self.image_batches) == 0:
