@@ -12,12 +12,12 @@
 #    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 #    License for the specific language governing permissions and limitations
 #    under the License.
+from unittest import mock
 import uuid
 
 from cursive import exception as cursive_exception
 import glance_store
 from glance_store._drivers import filesystem
-import mock
 import six
 from six.moves import http_client as http
 import webob
@@ -32,6 +32,7 @@ import glance.tests.utils as test_utils
 
 
 class Raise(object):
+
     def __init__(self, exc):
         self.exc = exc
 
@@ -40,6 +41,7 @@ class Raise(object):
 
 
 class FakeImage(object):
+
     def __init__(self, image_id=None, data=None, checksum=None, size=0,
                  virtual_size=0, locations=None, container_format='bear',
                  disk_format='rawr', status=None):
@@ -69,13 +71,14 @@ class FakeImage(object):
             return self.data[offset:offset + chunk_size]
         return self.data[offset:]
 
-    def set_data(self, data, size=None, backend=None):
+    def set_data(self, data, size=None, backend=None, set_active=True):
         self.data = ''.join(data)
         self.size = size
         self.status = 'modified-by-fake'
 
 
 class FakeImageRepo(object):
+
     def __init__(self, result=None):
         self.result = result
 
@@ -90,6 +93,7 @@ class FakeImageRepo(object):
 
 
 class FakeGateway(object):
+
     def __init__(self, db=None, store=None, notifier=None,
                  policy=None, repo=None):
         self.db = db
@@ -103,6 +107,7 @@ class FakeGateway(object):
 
 
 class TestImagesController(base.StoreClearingUnitTest):
+
     def setUp(self):
         super(TestImagesController, self).setUp()
 
@@ -157,6 +162,7 @@ class TestImagesController(base.StoreClearingUnitTest):
 
     def test_download_ok_when_get_image_location_forbidden(self):
         class ImageLocations(object):
+
             def __len__(self):
                 raise exception.Forbidden()
 
@@ -366,59 +372,6 @@ class TestImagesController(base.StoreClearingUnitTest):
                           self.controller.upload,
                           request, unit_test_utils.UUID2, 'YY', 2)
 
-    @mock.patch("glance.common.trust_auth.TokenRefresher")
-    def test_upload_with_trusts(self, mock_refresher):
-        """Test that uploading with registry correctly uses trusts"""
-        # initialize trust environment
-        self.config(data_api='glance.db.registry.api')
-        refresher = mock.MagicMock()
-        mock_refresher.return_value = refresher
-        refresher.refresh_token.return_value = "fake_token"
-        # request an image upload
-        request = unit_test_utils.get_fake_request()
-        request.environ['keystone.token_auth'] = mock.MagicMock()
-        request.environ['keystone.token_info'] = {
-            'token': {
-                'roles': [{'name': 'FakeRole', 'id': 'FakeID'}]
-            }
-        }
-        image = FakeImage('abcd')
-        self.image_repo.result = image
-        mock_fake_save = mock.Mock()
-        mock_fake_save.side_effect = [None, exception.NotAuthenticated, None]
-        temp_save = FakeImageRepo.save
-        # mocking save to raise NotAuthenticated on the second call
-        FakeImageRepo.save = mock_fake_save
-        self.controller.upload(request, unit_test_utils.UUID2, 'YYYY', 4)
-        # check image data
-        self.assertEqual('YYYY', image.data)
-        self.assertEqual(4, image.size)
-        FakeImageRepo.save = temp_save
-        # check that token has been correctly acquired and deleted
-        mock_refresher.assert_called_once_with(
-            request.environ['keystone.token_auth'],
-            request.context.tenant, ['FakeRole'])
-        refresher.refresh_token.assert_called_once_with()
-        refresher.release_resources.assert_called_once_with()
-        self.assertEqual("fake_token", request.context.auth_token)
-
-    @mock.patch("glance.common.trust_auth.TokenRefresher")
-    def test_upload_with_trusts_fails(self, mock_refresher):
-        """Test upload with registry if trust was not successfully created"""
-        # initialize trust environment
-        self.config(data_api='glance.db.registry.api')
-        mock_refresher().side_effect = Exception()
-        # request an image upload
-        request = unit_test_utils.get_fake_request()
-        image = FakeImage('abcd')
-        self.image_repo.result = image
-        self.controller.upload(request, unit_test_utils.UUID2, 'YYYY', 4)
-        # check image data
-        self.assertEqual('YYYY', image.data)
-        self.assertEqual(4, image.size)
-        # check that the token has not been updated
-        self.assertEqual(0, mock_refresher().refresh_token.call_count)
-
     def _test_upload_download_prepare_notification(self):
         request = unit_test_utils.get_fake_request()
         self.controller.upload(request, unit_test_utils.UUID2, 'YYYY', 4)
@@ -888,9 +841,9 @@ class TestImageDataSerializer(test_utils.BaseTestCase):
         def get_data(*args, **kwargs):
             raise exception.Forbidden()
 
-        self.stubs.Set(glance.api.policy.ImageProxy,
-                       'get_data',
-                       get_data)
+        self.mock_object(glance.api.policy.ImageProxy,
+                         'get_data',
+                         get_data)
         request = wsgi.Request.blank('/')
         request.environ = {}
         response = webob.Response()
