@@ -14,6 +14,7 @@
 #    under the License.
 
 import datetime
+from unittest import mock
 
 from oslo_serialization import jsonutils
 import webob
@@ -23,6 +24,7 @@ from glance.api.v2 import metadef_objects as objects
 from glance.api.v2 import metadef_properties as properties
 from glance.api.v2 import metadef_resource_types as resource_types
 from glance.api.v2 import metadef_tags as tags
+import glance.gateway
 from glance.tests.unit import base
 import glance.tests.unit.utils as unit_test_utils
 
@@ -395,6 +397,52 @@ class TestMetadefsControllers(base.IsolatedUnitTest):
             [self.assertTrue(property_name.startswith(rt.prefix)) for
              property_name in object.properties.keys()]
 
+    @mock.patch('glance.api.v2.metadef_namespaces.LOG')
+    def test_cleanup_namespace_success(self, mock_log):
+        fake_gateway = glance.gateway.Gateway(db_api=self.db,
+                                              notifier=self.notifier,
+                                              policy_enforcer=self.policy)
+        req = unit_test_utils.get_fake_request()
+        ns_factory = fake_gateway.get_metadef_namespace_factory(
+            req.context)
+        ns_repo = fake_gateway.get_metadef_namespace_repo(req.context)
+        namespace = namespaces.Namespace()
+        namespace.namespace = 'FakeNamespace'
+        new_namespace = ns_factory.new_namespace(**namespace.to_dict())
+        ns_repo.add(new_namespace)
+
+        self.namespace_controller._cleanup_namespace(ns_repo, namespace, True)
+
+        mock_log.debug.assert_called_with(
+            "Cleaned up namespace %(namespace)s ",
+            {'namespace': namespace.namespace})
+
+    @mock.patch('glance.api.v2.metadef_namespaces.LOG')
+    @mock.patch('glance.api.authorization.MetadefNamespaceRepoProxy.remove')
+    def test_cleanup_namespace_exception(self, mock_remove, mock_log):
+        mock_remove.side_effect = Exception(u'Mock remove was called')
+
+        fake_gateway = glance.gateway.Gateway(db_api=self.db,
+                                              notifier=self.notifier,
+                                              policy_enforcer=self.policy)
+        req = unit_test_utils.get_fake_request()
+        ns_factory = fake_gateway.get_metadef_namespace_factory(
+            req.context)
+        ns_repo = fake_gateway.get_metadef_namespace_repo(req.context)
+        namespace = namespaces.Namespace()
+        namespace.namespace = 'FakeNamespace'
+        new_namespace = ns_factory.new_namespace(**namespace.to_dict())
+        ns_repo.add(new_namespace)
+
+        self.namespace_controller._cleanup_namespace(ns_repo, namespace, True)
+
+        called_msg = 'Failed to delete namespace %(namespace)s.' \
+                     'Exception: %(exception)s'
+        called_args = {'exception': u'Mock remove was called',
+                       'namespace': u'FakeNamespace'}
+        mock_log.error.assert_called_with((called_msg, called_args))
+        mock_remove.assert_called_once_with(mock.ANY)
+
     def test_namespace_show_non_existing(self):
         request = unit_test_utils.get_fake_request()
         self.assertRaises(webob.exc.HTTPNotFound,
@@ -990,11 +1038,14 @@ class TestMetadefsControllers(base.IsolatedUnitTest):
         request = unit_test_utils.get_fake_request('/metadefs/namespaces/'
                                                    'Namespace3/'
                                                    'properties')
-        request.body = jsonutils.dump_as_bytes({'name': 'a' * 256})
+        request.body = jsonutils.dump_as_bytes({
+            'name': 'a' * 81, 'type': 'string', 'title': 'fake'})
 
-        self.assertRaises(webob.exc.HTTPBadRequest,
-                          self.property_deserializer.create,
-                          request)
+        exc = self.assertRaises(webob.exc.HTTPBadRequest,
+                                self.property_deserializer.create,
+                                request)
+        self.assertIn("Failed validating 'maxLength' in "
+                      "schema['properties']['name']", exc.explanation)
 
     def test_property_create_with_4byte_character(self):
         request = unit_test_utils.get_fake_request()
@@ -1175,6 +1226,16 @@ class TestMetadefsControllers(base.IsolatedUnitTest):
                           PROPERTY1, property)
         self.assertNotificationsLog([])
 
+    def test_property_update_with_overlimit_name(self):
+        request = unit_test_utils.get_fake_request()
+        request.body = jsonutils.dump_as_bytes({
+            'name': 'a' * 81, 'type': 'string', 'title': 'fake'})
+        exc = self.assertRaises(webob.exc.HTTPBadRequest,
+                                self.property_deserializer.create,
+                                request)
+        self.assertIn("Failed validating 'maxLength' in "
+                      "schema['properties']['name']", exc.explanation)
+
     def test_property_update_with_4byte_character(self):
         request = unit_test_utils.get_fake_request(tenant=TENANT3)
 
@@ -1344,15 +1405,38 @@ class TestMetadefsControllers(base.IsolatedUnitTest):
         self.assertEqual([], object.required)
         self.assertEqual({}, object.properties)
 
+    def test_object_create_invalid_properties(self):
+        request = unit_test_utils.get_fake_request('/metadefs/namespaces/'
+                                                   'Namespace3/'
+                                                   'objects')
+        body = {
+            "name": "My Object",
+            "description": "object1 description.",
+            "properties": {
+                "property1": {
+                    "type": "integer",
+                    "title": "property",
+                    "description": "property description",
+                    "test-key": "test-value",
+                }
+            }
+        }
+        request.body = jsonutils.dump_as_bytes(body)
+        self.assertRaises(webob.exc.HTTPBadRequest,
+                          self.deserializer.create,
+                          request)
+
     def test_object_create_overlimit_name(self):
         request = unit_test_utils.get_fake_request('/metadefs/namespaces/'
                                                    'Namespace3/'
                                                    'objects')
-        request.body = jsonutils.dump_as_bytes({'name': 'a' * 256})
+        request.body = jsonutils.dump_as_bytes({'name': 'a' * 81})
 
-        self.assertRaises(webob.exc.HTTPBadRequest,
-                          self.deserializer.create,
-                          request)
+        exc = self.assertRaises(webob.exc.HTTPBadRequest,
+                                self.deserializer.create,
+                                request)
+        self.assertIn("Failed validating 'maxLength' in "
+                      "schema['properties']['name']", exc.explanation)
 
     def test_object_create_duplicate(self):
         request = unit_test_utils.get_fake_request()
@@ -1505,6 +1589,15 @@ class TestMetadefsControllers(base.IsolatedUnitTest):
                           self.object_controller.update, request,
                           object, NAMESPACE1, OBJECT1)
 
+    def test_object_update_with_overlimit_name(self):
+        request = unit_test_utils.get_fake_request()
+        request.body = jsonutils.dump_as_bytes(
+            {"properties": {}, "name": "a" * 81, "required": []})
+        exc = self.assertRaises(webob.exc.HTTPBadRequest,
+                                self.deserializer.update, request)
+        self.assertIn("Failed validating 'maxLength' in "
+                      "schema['properties']['name']", exc.explanation)
+
     def test_object_update_conflict(self):
         request = unit_test_utils.get_fake_request(tenant=TENANT3)
 
@@ -1837,9 +1930,11 @@ class TestMetadefsControllers(base.IsolatedUnitTest):
     def test_tag_create_overlimit_name(self):
         request = unit_test_utils.get_fake_request()
 
-        self.assertRaises(webob.exc.HTTPBadRequest,
-                          self.tag_controller.create,
-                          request, NAMESPACE1, 'a' * 256)
+        exc = self.assertRaises(webob.exc.HTTPBadRequest,
+                                self.tag_controller.create,
+                                request, NAMESPACE1, 'a' * 81)
+        self.assertIn("Failed validating 'maxLength' in "
+                      "schema['properties']['name']", exc.explanation)
 
     def test_tag_create_with_4byte_character(self):
         request = unit_test_utils.get_fake_request()
@@ -1985,6 +2080,15 @@ class TestMetadefsControllers(base.IsolatedUnitTest):
                           self.tag_controller.update, request, tag,
                           NAMESPACE1, TAG1)
 
+    def test_tag_update_with_name_overlimit(self):
+        request = unit_test_utils.get_fake_request()
+        request.body = jsonutils.dump_as_bytes(
+            {"properties": {}, "name": "a" * 81, "required": []})
+        exc = self.assertRaises(webob.exc.HTTPBadRequest,
+                                self.deserializer.update, request)
+        self.assertIn("Failed validating 'maxLength' in "
+                      "schema['properties']['name']", exc.explanation)
+
     def test_tag_update_conflict(self):
         request = unit_test_utils.get_fake_request(tenant=TENANT3)
 
@@ -2016,3 +2120,16 @@ class TestMetadefsControllers(base.IsolatedUnitTest):
                           self.tag_controller.update, request, tag,
                           NAMESPACE4, TAG1)
         self.assertNotificationsLog([])
+
+
+class TestMetadefNamespaceResponseSerializers(base.IsolatedUnitTest):
+
+    def setUp(self):
+        super(TestMetadefNamespaceResponseSerializers, self).setUp()
+        self.serializer = namespaces.ResponseSerializer(schema={})
+        self.response = mock.Mock()
+        self.result = mock.Mock()
+
+    def test_delete_tags(self):
+        self.serializer.delete_tags(self.response, self.result)
+        self.assertEqual(204, self.response.status_int)
