@@ -20,8 +20,12 @@ import functools
 import os
 import shlex
 import shutil
+import signal
 import socket
 import subprocess
+import threading
+import time
+from unittest import mock
 
 from alembic import command as alembic_command
 import fixtures
@@ -29,8 +33,8 @@ from oslo_config import cfg
 from oslo_config import fixture as cfg_fixture
 from oslo_log.fixture import logging_error as log_fixture
 from oslo_log import log
-from oslo_serialization import jsonutils
-from oslotest import moxstubout
+from oslo_utils import timeutils
+from oslo_utils import units
 import six
 from six.moves import BaseHTTPServer
 from six.moves import http_client as http
@@ -40,16 +44,15 @@ import webob
 from glance.common import config
 from glance.common import exception
 from glance.common import property_utils
-from glance.common import timeutils
 from glance.common import utils
 from glance.common import wsgi
 from glance import context
 from glance.db.sqlalchemy import alembic_migrations
 from glance.db.sqlalchemy import api as db_api
-from glance.db.sqlalchemy import models as db_models
 from glance.tests.unit import fixtures as glance_fixtures
 
 CONF = cfg.CONF
+LOG = log.getLogger(__name__)
 try:
     CONF.debug
 except cfg.NoSuchOptError:
@@ -77,10 +80,9 @@ class BaseTestCase(testtools.TestCase):
         # the following policy tests
         config.parse_args(args=[])
         self.addCleanup(CONF.reset)
-        mox_fixture = self.useFixture(moxstubout.MoxStubout())
-        self.stubs = mox_fixture.stubs
-        self.stubs.Set(exception, '_FATAL_EXCEPTION_FORMAT_ERRORS', True)
+        self.mock_object(exception, '_FATAL_EXCEPTION_FORMAT_ERRORS', True)
         self.test_dir = self.useFixture(fixtures.TempDir()).path
+        self.test_dir2 = self.useFixture(fixtures.TempDir()).path
         self.conf_dir = os.path.join(self.test_dir, 'etc')
         utils.safe_mkdirs(self.conf_dir)
         self.set_policy()
@@ -139,9 +141,41 @@ class BaseTestCase(testtools.TestCase):
         """
         self._config_fixture.config(**kw)
 
+    def mock_object(self, obj, attr_name, *args, **kwargs):
+        """"Use python mock to mock an object attirbute
+
+        Mocks the specified objects attribute with the given value.
+        Automatically performs 'addCleanup' for the mock.
+        """
+        patcher = mock.patch.object(obj, attr_name, *args, **kwargs)
+        result = patcher.start()
+        self.addCleanup(patcher.stop)
+        return result
+
+    def delay_inaccurate_clock(self, duration=0.001):
+        """Add a small delay to compensate for inaccurate system clocks.
+
+        Some tests make assertions based on timestamps (e.g. comparing
+        'created_at' and 'updated_at' fields). In some cases, subsequent
+        time.time() calls may return identical values (python timestamps can
+        have a lower resolution on Windows compared to Linux - 1e-7 as
+        opposed to 1e-9).
+
+        A small delay (a few ms should be negligeable) can prevent such
+        issues. At the same time, it spares us from mocking the time
+        module, which might be undesired.
+        """
+
+        # For now, we'll do this only for Windows. If really needed,
+        # on Py3 we can get the clock resolution using time.get_clock_info,
+        # but at that point we may as well just sleep 1ms all the time.
+        if os.name == 'nt':
+            time.sleep(duration)
+
 
 class requires(object):
     """Decorator that initiates additional test setup/teardown."""
+
     def __init__(self, setup=None, teardown=None):
         self.setup = setup
         self.teardown = teardown
@@ -160,12 +194,17 @@ class requires(object):
 
 class depends_on_exe(object):
     """Decorator to skip test if an executable is unavailable"""
+
     def __init__(self, exe):
         self.exe = exe
 
     def __call__(self, func):
         def _runner(*args, **kw):
-            cmd = 'which %s' % self.exe
+            if os.name != 'nt':
+                cmd = 'which %s' % self.exe
+            else:
+                cmd = 'where.exe', '%s' % self.exe
+
             exitcode, out, err = execute(cmd, raise_error=False)
             if exitcode != 0:
                 args[0].disabled_message = 'test requires exe: %s' % self.exe
@@ -245,7 +284,8 @@ def fork_exec(cmd,
 
 def wait_for_fork(pid,
                   raise_error=True,
-                  expected_exitcode=0):
+                  expected_exitcode=0,
+                  force=True):
     """
     Wait for a process to complete
 
@@ -254,18 +294,52 @@ def wait_for_fork(pid,
     is raised.
     """
 
-    rc = 0
-    try:
-        (pid, rc) = os.waitpid(pid, 0)
-        rc = os.WEXITSTATUS(rc)
-        if rc != expected_exitcode:
-            raise RuntimeError('The exit code %d is not %d'
-                               % (rc, expected_exitcode))
-    except Exception:
-        if raise_error:
-            raise
+    # For the first period, we wait without being pushy, but after
+    # this timer expires, we start sending SIGTERM
+    term_timer = timeutils.StopWatch(5)
+    term_timer.start()
 
-    return rc
+    # After this timer expires we start sending SIGKILL
+    nice_timer = timeutils.StopWatch(7)
+    nice_timer.start()
+
+    # Process gets a maximum amount of time to exit before we fail the
+    # test
+    total_timer = timeutils.StopWatch(10)
+    total_timer.start()
+
+    while not total_timer.expired():
+        try:
+            cpid, rc = os.waitpid(pid, force and os.WNOHANG or 0)
+            if cpid == 0 and force:
+                if not term_timer.expired():
+                    # Waiting for exit on first signal
+                    pass
+                elif not nice_timer.expired():
+                    # Politely ask the process to GTFO
+                    LOG.warning('Killing child %i with SIGTERM' % pid)
+                    os.kill(pid, signal.SIGTERM)
+                else:
+                    # No more Mr. Nice Guy
+                    LOG.warning('Killing child %i with SIGKILL' % pid)
+                    os.kill(pid, signal.SIGKILL)
+                    expected_exitcode = signal.SIGKILL
+                time.sleep(1)
+                continue
+            LOG.info('waitpid(%i) returned %i,%i' % (pid, cpid, rc))
+            if rc != expected_exitcode:
+                raise RuntimeError('The exit code %d is not %d'
+                                   % (rc, expected_exitcode))
+            return rc
+        except ChildProcessError:
+            # Nothing to wait for
+            return 0
+        except Exception as e:
+            LOG.error('Got wait error: %s' % e)
+            if raise_error:
+                raise
+
+    raise RuntimeError('Gave up waiting for %i to exit!' % pid)
 
 
 def execute(cmd,
@@ -314,7 +388,11 @@ def execute(cmd,
     path_ext = [os.path.join(os.getcwd(), 'bin')]
 
     # Also jack in the path cmd comes from, if it's absolute
-    args = shlex.split(cmd)
+    if os.name != 'nt':
+        args = shlex.split(cmd)
+    else:
+        args = cmd
+
     executable = args[0]
     if os.path.isabs(executable):
         path_ext.append(os.path.dirname(executable))
@@ -473,7 +551,7 @@ def start_http_server(image_id, image_data):
                 self.send_response(http.OK)
                 self.send_header('Content-Length', str(len(fixture)))
                 self.end_headers()
-                self.wfile.write(fixture)
+                self.wfile.write(six.b(fixture))
                 return
 
             def do_HEAD(self):
@@ -499,84 +577,11 @@ def start_http_server(image_id, image_data):
     httpd = BaseHTTPServer.HTTPServer(server_address, handler_class)
     port = httpd.socket.getsockname()[1]
 
-    pid = os.fork()
-    if pid == 0:
-        httpd.serve_forever()
-    else:
-        return pid, port
+    thread = threading.Thread(target=httpd.serve_forever)
+    thread.daemon = True
+    thread.start()
 
-
-class RegistryAPIMixIn(object):
-
-    def create_fixtures(self):
-        for fixture in self.FIXTURES:
-            db_api.image_create(self.context, fixture)
-            with open(os.path.join(self.test_dir, fixture['id']),
-                      'wb') as image:
-                image.write(b"chunk00000remainder")
-
-    def destroy_fixtures(self):
-        db_models.unregister_models(db_api.get_engine())
-        db_models.register_models(db_api.get_engine())
-
-    def get_fixture(self, **kwargs):
-        fixture = {'name': 'fake public image',
-                   'status': 'active',
-                   'disk_format': 'vhd',
-                   'container_format': 'ovf',
-                   'visibility': 'public',
-                   'size': 20,
-                   'checksum': None}
-        if 'is_public' in kwargs:
-            fixture.pop('visibility')
-        fixture.update(kwargs)
-        return fixture
-
-    def get_minimal_fixture(self, **kwargs):
-        fixture = {'name': 'fake public image',
-                   'visibility': 'public',
-                   'disk_format': 'vhd',
-                   'container_format': 'ovf'}
-        if 'is_public' in kwargs:
-            fixture.pop('visibility')
-        fixture.update(kwargs)
-        return fixture
-
-    def get_extra_fixture(self, id, name, **kwargs):
-        created_at = kwargs.pop('created_at', timeutils.utcnow())
-        updated_at = kwargs.pop('updated_at', created_at)
-        return self.get_fixture(
-            id=id, name=name, deleted=False, deleted_at=None,
-            created_at=created_at, updated_at=updated_at,
-            **kwargs)
-
-    def get_api_response_ext(self, http_resp, url='/images', headers=None,
-                             body=None, method=None, api=None,
-                             content_type=None):
-        if api is None:
-            api = self.api
-        if headers is None:
-            headers = {}
-        req = webob.Request.blank(url)
-        for k, v in six.iteritems(headers):
-            req.headers[k] = v
-        if method:
-            req.method = method
-        if body:
-            req.body = body
-        if content_type == 'json':
-            req.content_type = 'application/json'
-        elif content_type == 'octet':
-            req.content_type = 'application/octet-stream'
-        res = req.get_response(api)
-        self.assertEqual(res.status_int, http_resp)
-        return res
-
-    def assertEqualImages(self, res, uuids, key='images', unjsonify=True):
-        images = jsonutils.loads(res.body)[key] if unjsonify else res
-        self.assertEqual(len(images), len(uuids))
-        for i, value in enumerate(uuids):
-            self.assertEqual(images[i]['id'], value)
+    return thread, httpd, port
 
 
 class FakeAuthMiddleware(wsgi.Middleware):
@@ -699,3 +704,69 @@ def is_sqlite_version_prior_to(major, minor):
     import sqlite3
     tup = sqlite3.sqlite_version_info
     return tup[0] < major or (tup[0] == major and tup[1] < minor)
+
+
+def start_standalone_http_server():
+    def _get_http_handler_class():
+        class StaticHTTPRequestHandler(BaseHTTPServer.BaseHTTPRequestHandler):
+            def do_GET(self):
+                data = b"Hello World!!!"
+                self.send_response(http.OK)
+                self.send_header('Content-Length', str(len(data)))
+                self.end_headers()
+                self.wfile.write(data)
+                return
+
+        return StaticHTTPRequestHandler
+
+    server_address = ('127.0.0.1', 0)
+    handler_class = _get_http_handler_class()
+    httpd = BaseHTTPServer.HTTPServer(server_address, handler_class)
+    port = httpd.socket.getsockname()[1]
+
+    thread = threading.Thread(target=httpd.serve_forever)
+    thread.daemon = True
+    thread.start()
+
+    return thread, httpd, port
+
+
+class FakeData(object):
+    """Generate a bunch of data without storing it in memory.
+
+    This acts like a read-only file object which generates fake data
+    in chunks when read() is called or it is used as a generator. It
+    can generate an arbitrary amount of data without storing it in
+    memory.
+
+    :param length: The number of bytes to generate
+    :param chunk_size: The chunk size to return in iteration mode, or when
+                       read() is called unbounded
+
+    """
+    def __init__(self, length, chunk_size=64 * units.Ki):
+        self._max = length
+        self._chunk_size = chunk_size
+        self._len = 0
+
+    def read(self, length=None):
+        if length is None:
+            length = self._chunk_size
+
+        length = min(length, self._max - self._len)
+
+        self._len += length
+        if length == 0:
+            return b''
+        else:
+            return b'0' * length
+
+    def __iter__(self):
+        return self
+
+    def __next__(self):
+        r = self.read()
+        if len(r) == 0:
+            raise StopIteration()
+        else:
+            return r
